#!/usr/bin/php
<?php
/**
 * WMS automatic upgrade script.
 * Created by Alex Soloviov.
 */

$basepath = '/var/www';
require_once($basepath.'/autoload.php');
require_once('lang/' . ((isset($_SESSION['language'])) ? $_SESSION['language'] : 'english') . '.inc.php');

$diContainer = new \Wildix\DependencyInjection\Container();

/** @var \Wildix\System\Upgrade\AutoUpgrade $upgradeService */
$upgradeService = $diContainer->get(\Wildix\System\Upgrade\AutoUpgrade::class);

/** @var \Wildix\System\Upgrade\UpgradeSettings $upgradeSettings */
$upgradeSettings = $diContainer->get(\Wildix\System\Upgrade\UpgradeSettings::class);

/** @var \Logger\Log $logger */
$logger = $diContainer->get(\Logger\Log::class);
$logger->setLogLevel(\Logger\Log::DEBUG);

$repo             = null;
$email            = '';
$isAllowedUpgrade = false;

$fromPortal = (!empty($argv[1]) && $argv[1] == 'from_portal') ? true : false;

if ($fromPortal) {
    try {
        $portalService = $diContainer->get(Wildix\Services\Portal::class);
        $portalParams  = $portalService->getUpgradeParameters();
        $logger->info(__LINE__, sprintf('Upgrade parameters from portal: %s', json_encode($portalParams)));
    } catch (\Exception $e) {
        $logger->error(__LINE__, $e->getMessage() . ' RETURN CODE: ' . $e->getCode(), 'AUTOUPGRADE');
        throw new \Exception($e->getMessage(), $e->getCode());
    }

    if (!empty($portalParams['repo'])) {
        $repo             = $portalParams['repo'];
        $email            = $portalParams['email'] ?? '';
        $isAllowedUpgrade = true;
    }
} else {
    $localParams = $upgradeSettings->getAutoUpgradeConfig();

    if (!empty($localParams['repository'])) {
        $repo             = $localParams['repository'];
        $email            = $localParams['email'] ?? '';
        $isAllowedUpgrade = true;
    }
}

if ($isAllowedUpgrade) {
    $logger->info(__LINE__, 'Start automatic upgrade');
    if (!$upgradeService->makeAutoUpgrade($repo, $email)) {
        $logger->error(__LINE__, 'Automatic upgrade finished with errors');
    }
    $logger->info(__LINE__, 'Finished automatic upgrade');
} else {
    $logger->info(__LINE__, 'PBX automatic upgrade not allowed');
}
