#!/bin/bash

# set env
LC_ALL="en_US.UTF-8"
PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin

# Init variables
DPKG_RECOVERED=0
CHECK_RW='/rw.tmp'
CONFIG='/rw2/var/recovery.conf'

# init log file
ERR_FILE='/rw2/var/recovery.err'
LOG_FILE='/rw2/var/recovery.log'

IsItChroot() {

  if [ "$(grep -i "initctl" /proc/mounts)" ]; then
    echo "YES"
  elif [ "$(grep -iE "QEMU Virtual CPU" /proc/cpuinfo)" ]; then
    echo "YES"
  fi

}

finding_corrupted_files() {

  local ERRORS="$(mktemp)"; local GET_ERRORS

  find / -type f  ! -path "/proc*" ! -path "/sys*" ! -path "/dev*" 2> "$ERRORS" > /dev/null
  grep -c "Input/output error" $ERRORS
  rm "$ERRORS"

}

check_filesystem() {

  echo -e "\nCHECK_FILESYSTEM: Checking corrupted files..."

  GET_CORRUPT_FILES="$(finding_corrupted_files "/")"
  GET_COUNT_FSCK="$(awk '/tryfsck/{print $2}' $CONFIG)"
  GET_RMA="$(awk '/rma/{print $2}' $CONFIG)"

  if [ "$GET_CORRUPT_FILES" -ge "1" ]; then
    echo "CHECK_FILESYSTEM: filesystem is corrupted!"
      if [ "$GET_COUNT_FSCK" -lt "1" ]; then
        echo "CHECK_FILESYSTEM: running 2nd fsck and reboot ..."
        ((GET_COUNT_FSCK++))

        sed -i "s/^tryfsck.*/tryfsck $GET_COUNT_FSCK/" $CONFIG
          grep -E "/ |/rw2|/boot" /etc/fstab|awk '{print $1}'|sed -e 's/\/dev\///' | while read line; do
            sed -i "/$line/s/0$/1/g" /etc/fstab
          done
        touch /forcefsck
        echo -e "\n---\nreboot ...\n---\n" && reboot && exit

      else
        echo -e "\n---\nERROR: This PBX after 2nd fsck can't be recoverable... (exit)\n---\n"
        sed -i 's/^rma 0$/rma 1/' $CONFIG
        sed -i 's/^recovery 1$/recovery 0/' $CONFIG
        sed -i 's/^showlog 1$/showlog 0/' $CONFIG
        exit
      fi
  else
    sed -i 's/^rma 1$/rma 0/' $CONFIG
    sed -i 's/^tryfsck.*/tryfsck 0/' $CONFIG
    echo "CHECK_FILESYSTEM: done"
  fi

}

check_backup() {

  local BACKUP_FILE="$(ls -t /mnt/backups/*.tar.gz 2> /dev/null | head -n 1)"

  echo -e "\nCHECK_BACKUP: Checking backup..."
    if [ -f "$BACKUP_FILE" ]; then
      [ "$(($(date +%s) - $(stat -c %Y $BACKUP_FILE)))" -ge "3600" ] && echo "CHECK_BACKUP: create new backup..." && timeout 60 /usr/sbin/wms_backup_create || echo "CHECK_BACKUP: backup is already done!"
    else
      echo "CHECK_BACKUP: create new backup..." && timeout 60 /usr/sbin/wms_backup_create
    fi
  echo "CHECK_BACKUP: done"

}

check_rw() {

  if touch ${1}/rw.tmp &>/dev/null; then
    rm -f ${1}/rw.tmp
    echo "CHECK_RW: ${1} is rw mode..." >> $LOG_FILE
    return 0
  else
    echo "CHECK_RW: re-mount ${1} to rw mode..." >> $LOG_FILE
    mount -o remount,rw ${1} >> $LOG_FILE
    [ "$?" -ne "0" ] && echo "CHECK_RW: Cannot re-mount ${1} partition with rw option" >> $LOG_FILE && exit || return 1
  fi

}

ldap_close_processes() {

  # clean up process
  for PROCESS in $(lsof | grep "/rw2/var/lib/ldap" | awk '{print $2}' | sort -u); do
    echo "LDAP_CLOSE_PROCESSES: $PROCESS - kill"
    kill -9 $PROCESS
  done

}

ldap_recover_conf() {

  if [ -f "/rw2/var/backup/slapd.tgz" ]; then
    echo "LDAP_RECOVER_CONF: recovering from (/rw2/var/backup/slapd.tgz)"
    ldap_close_processes

      # clean up shared memory
      for addr in $(ipcs | awk '/openldap|0x0000000/{print $2}'); do
        ipcrm -m $addr
      done

    rm -rf /etc/ldap/*
    tar xzf /rw2/var/backup/slapd.tgz -C / &> /dev/null
    /etc/init.d/slapd restart
  else
    echo "LDAP_RECOVER_CONF: no backup file (/rw2/var/backup/slapd.tgz)"
  fi

}

ldap_reset_db() {

  ldap_close_processes
  /usr/sbin/ldap_reset reset

}

ldap_recover_db() {

  local ldifFile=''
  local serial="$(/usr/sbin/serial)"

    if [ "$(ls -t /mnt/backups/ | grep "tar.gz" -c)" -ne "0" ]; then
      backupFile="$(ls -t /mnt/backups/ | grep "tar.gz" | head -n 1)"
      tar xf /mnt/backups/$backupFile -C /tmp/ ldapcat.ldif

        if [ -s "/tmp/ldapcat.ldif" ] && [ -s "/backups/ldap_backup.ldiff" ]; then
          [ "/tmp/ldapcat.ldif" -nt "/backups/ldap_backup.ldiff" ] && ldifFile='/tmp/ldapcat.ldif' || ldifFile='/backups/ldap_backup.ldiff'
        else
          if [ -s "/tmp/ldapcat.ldif" ]; then
            ldifFile='/tmp/ldapcat.ldif'
          elif [ -s "/backups/ldap_backup.ldiff" ]; then
            ldifFile='/backups/ldap_backup.ldiff'
          fi
        fi

      timeout 30 slapcat &> /dev/null

        if [ "$?" -ne "0" ] || [ "$(timeout 30 slapcat | grep -c "$serial")" -eq "0" ]; then
          [ "$ldifFile" ] && /usr/sbin/ldap_reset backup $ldifFile 2>$ERR_FILE || echo "LDAP_RECOVER_DB: Not available ldap backup. Not correct files"
        fi
    else
      echo "LDAP_RECOVER_DB: Not available ldap backup."
    fi

}

fix_ldap() {

  echo -e "\nFIX_LDAP: Checking LDAP configuration ..."

  local DB_CONFIG='/rw2/var/lib/ldap/DB_CONFIG'
  local DB_DIR='/rw2/var/lib/ldap/'

  # Fix LDAP configuration
  slaptest -u -Q -F /etc/ldap/slapd.d/ &> /dev/null
    if [ "$?" -ne "0" ]; then
      echo "FIX_LDAP: Restore LDAP config files"
      ldap_recover_conf
    fi

  # Fix LDAP database

  # Always remove DB_CONFIG. It will be re-created on slapd start.
  [ -f "$DB_CONFIG" ] && rm -f "$DB_CONFIG"

  # Checking for the openldap folder
  [ ! -d "$DB_DIR" ] && mkdir "$DB_DIR" && chown -R openldap:openldap "$DB_DIR"

  # Check if admin are present in ldap otherwise - restore ldap to default
  timeout 30 ldapsearch -xLLL -D "cn=admin,dc=wildix" -b "dc=wildix" -w "wildix" base &> /dev/null
  CHECK_LDAP="$?"
    if [ "$CHECK_LDAP" -eq "49" ] || [ "$CHECK_LDAP" -eq "255" ]; then
      echo "FIX_LDAP: Restore LDAP db"
      ldap_recover_db

      # double check. if not recovered then reset to defaults
      timeout 30 ldapsearch -xLLL -D "cn=admin,dc=wildix" -b "dc=wildix" -w "wildix" base &> /dev/null
      CHECK_LDAP="$?"
        if [ "$CHECK_LDAP" -eq "49" ] || [ "$CHECK_LDAP" -eq "255" ]; then
          echo "FIX_LDAP: Reset LDAP to default"
          ldap_reset_db
        fi
    fi

  # sync pass between system and ldap
  /usr/sbin/wms_auth_sync &> /dev/null

  echo "FIX_LDAP: done"

}

fix_config_files() {

  echo -e "\nFIX_CONFIG_FILES: Checking config files ..."

  ## /rw2/etc/pbx/net.conf
  if [ ! "$(grep -s STARTUP /rw2/etc/pbx/net.conf)" ]; then
    echo "FIX_CONFIG_FILES: fixing (/rw2/etc/pbx/net.conf)"
    rm -f /rw2/etc/pbx/net.conf
    cat << 'EOF' > /rw2/etc/pbx/net.conf
STARTUP=0
SRV_IP=127.0.0.1
CLIENT_IP=5.15.0.1
PORT=1194
EOF
  fi

  ## /rw2/etc/wms.conf
  if [ ! -s "/rw2/etc/wms.conf" ]; then
    echo "FIX_CONFIG_FILES: fixing (/rw2/etc/wms.conf)"
    cat << 'EOF' > /rw2/etc/wms.conf

[Settings]
basepath="/var/www/"
extension_rec=""

EOF
  fi

  ## /rw2/etc/apt.conf
  if [ ! -s "/rw2/etc/apt.conf" ]; then
    echo "FIX_CONFIG_FILES: fixing (/rw2/etc/apt.conf)"
    cat << 'EOF' > /rw2/etc/apt.conf
DPkg
{
    // Old conffiles without prompt
    Options {"--force-confold";}
}
EOF
  fi
  echo "FIX_CONFIG_FILES: done"

}

dpkg_recover() {

  if [ -f "/rw2/var/backup/dpkg.tgz" ]; then
    echo "DPKG_RECOVER: recovering from (/rw2/var/backup/dpkg.tgz)"
    rm -rf /rw2/var/lib/dpkg
    tar xzf /rw2/var/backup/dpkg.tgz -PC /
    DPKG_RECOVERED="1"
  else
    echo "DPKG_RECOVER: error no backup (/rw2/var/backup/dpkg.tgz)"
  fi

}

fix_dpkg() {

  echo -e "\nFIX_DPKG: Checking dpkg database..."

    [ ! -f "/var/lib/dpkg/available" ] && dpkg_recover
    [ ! -f "/var/lib/dpkg/diversions" ] && dpkg_recover
    [ ! -f "/var/lib/dpkg/statoverride" ] && dpkg_recover
    [ ! -f "/var/lib/dpkg/status" ] && dpkg_recover
    [ ! -d "/var/lib/dpkg/alternatives" ] && dpkg_recover
    [ ! -d "/var/lib/dpkg/info" ] && dpkg_recover
    [ ! -d "/var/lib/dpkg/parts" ] && dpkg_recover
    [ ! -d "/var/lib/dpkg/triggers" ] && dpkg_recover
    [ ! -d "/var/lib/dpkg/updates" ] && dpkg_recover

    if [ "$DPKG_RECOVERED" != "1" ]; then
      for PKG in $(dpkg --get-selections | grep -w install | awk '{print $1}'); do
        dpkg -L $PKG &> /dev/null
        [ "$?" -ne "0" ] && dpkg_recover && break
      done
    fi

    /usr/bin/dpkg --force-confold --configure -a 2> /dev/null
    [ "$?" -ne "0" ] && echo "FIX_DPKG: Checking config - failed!" || echo "FIX_DPKG: Checking config - OK"

    # removing deleted packages
    local REMOVING_PKG=$(dpkg --get-selections | grep -w deinstall | cut -f1)
    [ "$REMOVING_PKG" ] && echo -e "FIX_DPKG: removing deleted packages...\n" && dpkg --purge $REMOVING_PKG 2> $ERR_FILE

    echo "FIX_DPKG: done"

}

fix_wmsdb() {

  echo -e "\nFIX_WMSDB: Checking wmsdb..."

  local DB_FILE='/rw2/var/lib/wmsdb.d/wmsdb'
  local DB_FILE_TMP="/rw2/var/lib/wmsdb.d/wmsdb_$(date +%Y%m%d%H%M)"

    if [ "$(sqlite3 $DB_FILE '.ta' 2> /dev/null | wc -w)" -lt "20" ]; then
      [ -f "$DB_FILE" ] && rm "$DB_FILE"
      echo "FIX_WMSDB: $DB_FILE corrupt. Trying to recover..."

        for WMSDB_BACKUP in $(ls -lt /rw2/var/lib/wmsdb.d/wmsdb* | awk '{print $8}'); do
          if [ "$(file $WMSDB_BACKUP | grep "compressed")" ]; then
            /bin/zcat $WMSDB_BACKUP | sqlite3 "$DB_FILE_TMP"
            WMSDB_BACKUP="$DB_FILE_TMP"
          fi
          if [ "$(sqlite3 $WMSDB_BACKUP '.ta' 2> /dev/null | wc -w)" -lt "20" ]; then
            rm "$WMSDB_BACKUP"
            echo "FIX_WMSDB: $WMSDB_BACKUP corrupt. Trying to recover..."
          else
            cp $WMSDB_BACKUP $DB_FILE && chown wms:wms $DB_FILE
            [ -f "$DB_FILE_TMP" ] && rm $DB_FILE_TMP
            echo "FIX_WMSDB: done"
            break
          fi
        done

      [ ! -f "$DB_FILE" ] && echo "FIX_WMSDB: not exist. Using default one."
    else
      echo "FIX_WMSDB: done"
    fi

}




fix_conffiles() {

  if [ -f "/var/lib/dpkg/info/${1}.conffiles" ]; then
      while read CONF_FILE; do
        if [ ! -f "$CONF_FILE" ]; then
          [ ! -d "/tmp/package" ] && mkdir -p /tmp/package && dpkg -x /var/cache/apt/archives/${1}*.deb /tmp/package 1> /dev/null 2> $ERR_FILE
          [ -f "/tmp/package${CONF_FILE}" ] && echo -e "\n---\nFIX_CONFFILES: Recovered conffile ($CONF_FILE) using default from package.\n---\n" && cp /tmp/package${CONF_FILE} $CONF_FILE || echo -e "\n---\nFIX_CONFFILES: not found - ($CONF_FILE)\n---\n"
        fi
      done < /var/lib/dpkg/info/${1}.conffiles
    rm -rf /tmp/package
  fi

}

update_pkg() {

  echo -e "\nUPDATE_PKG: Launching..."

  # add update pakages
  >$PKGS_FILE
  for PKG in $(/usr/bin/apt-get dist-upgrade --just-print | awk '/Inst /{print $2}'); do
    echo "$PKG" >> $PKGS_FILE
  done

  for PKG_UPDATE in $(sort -u $PKGS_FILE | sed ':a;N;$!ba;s/\n/ /g'); do
    timeout 120 sudo DEBIAN_FRONTEND=noninteractive /usr/bin/apt-get -y --force-yes --allow-unauthenticated -q -c /rw2/etc/apt.conf install $PKG_UPDATE 2> $ERR_FILE
    fix_conffiles "$PKG_UPDATE"
    [ "$?" -ne "0" ] && echo -e "\n---\nUPDATE_PKG: ($PKG_UPDATE) failed!\n---\n" || echo -e "\n---\nUPDATE_PKG: ($PKG_UPDATE) OK\n---\n"
    rm -f /var/cache/apt/archives/*.deb
  done

  echo -e "UPDATE_PKG: done"

}

reinstall_pkg() {

  FAIL_INSTALL_PKG="0"

  echo -e "\nREINSTALL_PKG: Launching..."

  LIST_OF_PAKAGES="
  wms-interface-sqlite3
  lua-modules-wms
  slapd
  php5-cgi
  php5-cli
  php5-common
  wildix-cw
  sipproxy
  apt
  ejabberd
  lighttpd-fastcgi-srv-lua
  wildix-con
  dnsmasq
  ntpdate
  memcached
  freeradius
  wildix-sslh
  "

  # add standart pakages
  for PKG in $LIST_OF_PAKAGES; do
    echo "$PKG" >> $PKGS_FILE
  done

  # fix rcX.d directory
  for i in 1 2 3 4 5 6 S; do
    [ ! -d "/etc/rc${i}.d" ] && echo "REINSTALL_PKG: fix rc${i}.d directory..." && mkdir "/etc/rc${i}.d"
  done

  # ntp reload
  /etc/init.d/ntp stop && /usr/sbin/ntpdate-debian && /etc/init.d/ntp start

  # Always reinstall basic packages
  echo "REINSTALL_PKG: Re-installing packages..."

  /usr/bin/apt-get clean 2> $ERR_FILE 1> /dev/null
  /usr/bin/apt-get update 2> $ERR_FILE 1> /dev/null
  timeout 120 /usr/bin/apt-get -y --force-yes --allow-unauthenticated -q -c /rw2/etc/apt.conf autoclean 2> $ERR_FILE 1> /dev/null
  timeout 120 /usr/bin/apt-get -y --force-yes --allow-unauthenticated -q -c /rw2/etc/apt.conf autoremove 2> $ERR_FILE 1> /dev/null

    for PKG_RECOVERY in $(sort -u $PKGS_FILE | sed ':a;N;$!ba;s/\n/ /g'); do
      timeout 120 sudo DEBIAN_FRONTEND=noninteractive /usr/bin/apt-get -y --force-yes --allow-unauthenticated -q -c /rw2/etc/apt.conf install --reinstall $PKG_RECOVERY 2> $ERR_FILE
      STATUS_RECOVERY="$?"
      fix_conffiles "$PKG_RECOVERY"

        if [ "$STATUS_RECOVERY" -ne "0" ]; then
          echo "REINSTALL_PKG: Re-installation of $PKG_RECOVERY package failed. Trying to fix..."
          timeout 120 sudo DEBIAN_FRONTEND=noninteractive /usr/bin/apt-get -y --force-yes --allow-unauthenticated -q -c /rw2/etc/apt.conf -f install
          [ "$?" -ne "0" ] && ((FAIL_INSTALL_PKG++)) && echo -e "\n---\nREINSTALL_PKG: ($PKG_RECOVERY) failed!\n---\n" || echo -e "\n---\nREINSTALL_PKG: ($PKG_RECOVERY) OK\n---\n"
        else
          echo -e "\n---\nREINSTALL_PKG: ($PKG_RECOVERY) OK\n---\n"
        fi

      rm -f /var/cache/apt/archives/*.deb
    done

  timeout 120 /usr/bin/apt-get -y --force-yes --allow-unauthenticated -q -c /rw2/etc/apt.conf autoremove 2> $ERR_FILE 1> /dev/null
  timeout 300 sudo DEBIAN_FRONTEND=noninteractive /usr/bin/apt-get -y --force-yes --allow-unauthenticated -q -c /rw2/etc/apt.conf -f install

  echo "REINSTALL_PKG: done"

}

# select mode
case "$1" in
  check)
    CORRUPTED_FILES="$(mktemp)"
    find / -type f  ! -path "/proc*" ! -path "/sys*" ! -path "/dev*" 2> "$CORRUPTED_FILES" > /dev/null
    echo -e "CORRUPTED FILES:\n$(grep "Input/output error" $CORRUPTED_FILES | tr -d "\`\:\'" | awk '{print $2" - "$3" "$4}')\n---\ncount: $(grep -c "Input/output error" $CORRUPTED_FILES)"
    rm -f "$CORRUPTED_FILES"
  ;;
  *)
    # check if recovery needed
    if [ "$(awk '/recovery/{print $2}' $CONFIG)" -eq "1" ]; then

      PKGS_FILE="$(mktemp)"

      [ -f "$ERR_FILE" ] && [ "$(($(date +%s) - $(stat -c %Y $ERR_FILE)))" -ge "3600" ] && >$ERR_FILE
      [ -f "$LOG_FILE" ] && [ "$(($(date +%s) - $(stat -c %Y $LOG_FILE)))" -ge "3600" ] && >$LOG_FILE

      echo "Starting Recovery procedure."
      echo -e "\n---\nRecovery started is [$(date +"%Y-%m-%d_%H-%M")]\n--\n" >> $LOG_FILE

      # init new variables
      [ ! "$(awk '/tryfsck/{print $2}' $CONFIG)" ] && echo "tryfsck 0" >> $CONFIG
      [ ! "$(awk '/rma/{print $2}' $CONFIG)" ] && echo "rma 0" >> $CONFIG

      # mount usb for check latest ldap backup
      /usr/sbin/wmsmount &> /dev/null

      # check rw on /,/rw2
      check_rw "/"
      check_rw "/rw2"

      # fix permissions
      chmod 0755 /etc/init.d/* >> $LOG_FILE

      # stop monit
      [ "$(ps aux | grep "[m]onit")" ] && /etc/init.d/monit stop >> $LOG_FILE

      # runs functions
      [ ! "$(IsItChroot)" ] && check_filesystem >> $LOG_FILE || echo -e "\nCHECK_FILESYSTEM: is it chroot... (skip)"
      check_backup >> $LOG_FILE
      fix_config_files >> $LOG_FILE
      fix_wmsdb >> $LOG_FILE
      fix_dpkg >> $LOG_FILE
      fix_ldap >> $LOG_FILE
      reinstall_pkg >> $LOG_FILE
      # not completely finished
      #update_pkg >> $LOG_FILE

      if [ "$FAIL_INSTALL_PKG" -ge "15" ]; then
        sed -i 's/^rma 0$/rma 1/' $CONFIG
        echo -e "\n---\nFailed recovery. [$(date +"%Y-%m-%d_%H-%M")]\n---\n" >> $LOG_FILE
      else
        echo -e "\n---\nYour system is OK now. [$(date +"%Y-%m-%d_%H-%M")]\n---\n" >> $LOG_FILE
      fi

      # clean-up
      rm -f $PKGS_FILE
      sed -i 's/^recovery 1$/recovery 0/' $CONFIG
      sed -i 's/^showlog 1$/showlog 0/' $CONFIG
      [ ! "$(IsItChroot)" ] && echo "reboot..." || echo "reboot..." >> $LOG_FILE
      sync && reboot

    fi
  ;;
esac