#!/bin/bash

LDAP_SYNC_OUT=/tmp/ldap_slave_sync.out
PID_PROVIDER_DISABLE_MONITORING=$!
SUCCESS_ATTEMPTS=0
MASTER_READ_FAILS=0
MASTER_PARTIAL_MATCHES=0
ADMIN_SYNC_FAILS=0


__set_monit(){

	local service="$1"
	local set="$2"
	local status="$(monit status | grep -A 1 "$service" | grep -o 'Not monitored')"


	if [ "$set" == 'off' ] && [ "$status" != 'Not monitored' ]; then
		monit unmonitor "$service"
	elif [ "$set" == 'on' ] && [ "$status" == 'Not monitored' ]; then
		monit monitor "$service"
	fi

}


__timestamp(){

	date +"%s"

}


# enable monitoring and disable ldap memory mod.
__doExit(){

	printf "state:%s;users_expected:%d;users_actual:%d;" "completed" "$USERS_1" "$USERS_2" > $LDAP_SYNC_OUT
	sed -i '/olcDbNoSync.*/d;/^# CRC32*/d' /etc/ldap/slapd.d/cn\=config/olcDatabase\=\{1\}mdb.ldif
	kill $PID_PROVIDER_DISABLE_MONITORING
	wait $PID_PROVIDER_DISABLE_MONITORING 2> /dev/null

	## memory patch <<
	if [ -d '/rw/ldap_sync' ]; then
		/bin/systemctl stop slapd || true
		rm -rf /rw2/var/lib/ldap && mkdir -p /rw2/var/lib/ldap && cp -a /rw/ldap_sync/* /rw2/var/lib/ldap/
		chown openldap:openldap -R /rw2/var/lib/ldap
		rm -rf /rw/ldap_sync
		systemctl start monit || true
	fi
	## <<

	/bin/systemctl restart slapd || true
	/bin/systemctl restart callweaver || true
	/bin/systemctl restart kamailio || true
	__set_monit 'ldap' 'on'
	__set_monit 'sipproxy' 'on'
	exit 0

}


__toLog(){

	msg="$1"
	logger -t "monitor_replication" --id="$$" -p local0.info "$msg"

}


__isAdminSynced(){

	local local_uid=$(slapcat -a '(&(objectClass=wPerson)(cn=admin)(userKind=admin))' | awk -F ': ' '$1 == "entryUUID" { print $2; }')
	local remote_uid=$(timeout 30 ldapsearch -xLLL -h 5.15.0.1 -D "cn=admin,dc=wildix" -b "dc=wildix" -w "wildix" "(&(objectClass=wPerson)(cn=admin)(userKind=admin))" entryUUID|grep -v dn| awk "{print \$2}"|head -n1)

	[ "$local_uid" = "$remote_uid" ] && echo yes

}


START_TIME=$(__timestamp)
/bin/systemctl stop kamailio || true

printf "state:%s;users_expected:%d;users_actual:%d;" "in progress" 0 0 > $LDAP_SYNC_OUT
__toLog "LDAP Replication started at: $START_TIME"


## go to replication (god give me a power)
while true; do

	CURR_TIME=$(__timestamp)
	if [ "$(($CURR_TIME - $START_TIME))" -gt "$((6 * 60 * 60 * 1))" ]; then
		__toLog "Replication timeout 6 hours reached and exiting"
		__doExit
	fi

	__set_monit 'ldap' 'off'
	__set_monit 'sipproxy' 'off'

	USERS_1="$(slapcat | grep -c 'dn: uid=')"
	sleep 5
	USERS_2="$(slapcat | grep -c 'dn: uid=')"

	__toLog "Users count: $USERS_2"
	printf "state:%s;users_expected:%d;users_actual:%d;" "in progress" "$USERS_1" "$USERS_2" > $LDAP_SYNC_OUT

	if [ "$USERS_1" -eq "$USERS_2" ]; then
		((SUCCESS_ATTEMPTS++))
		__toLog "Replication stop. Check # $SUCCESS_ATTEMPTS"

		if [ $SUCCESS_ATTEMPTS -eq 5 ]; then
			# when replication stopped, checking is admin synced
			if [ "$(__isAdminSynced)" != 'yes' ]; then
				# admin is not synced stile
				SUCCESS_ATTEMPTS=0
				((ADMIN_SYNC_FAILS++))

				if [ $ADMIN_SYNC_FAILS -eq 5 ]; then
					__toLog "Failed to sync admin. Exiting"
					__doExit
				fi
				continue
			fi

			# when replication stopped, read users's count form master
			MASTER_USERS_MIDDLE="$(timeout 60 ldapsearch -xLLL -h 5.15.0.1 -D 'cn=admin,dc=wildix' -b 'dc=wildix' -w 'wildix' "(objectClass=wPerson)" dn | grep -c 'dn: uid=')"
			__toLog "Users count on Primary PBX: $MASTER_USERS_MIDDLE"

			if [ "$MASTER_USERS_MIDDLE" -eq "$USERS_2" ]; then
				__toLog "Replication stopped with 100% match and exiting"
				__doExit
			elif [ $MASTER_USERS_MIDDLE -eq 0 ]; then
				# master somehow is not available, wait if it's available
				SUCCESS_ATTEMPTS=0
				((MASTER_READ_FAILS++))
				if [ $MASTER_READ_FAILS -eq 5 ]; then
					__toLog "Failed to read from master. Exiting"
					__doExit
				fi
				sleep 30
			else
				SUCCESS_ATTEMPTS=0
				MASTER_READ_FAILS=0
				((MASTER_PARTIAL_MATCHES++))
				if [ $MASTER_PARTIAL_MATCHES -eq 5 ]; then
					# master is not available for too long
					__toLog "Replication stopped with partial match and exiting"
					__doExit
				fi
				sleep 30
			fi
		fi
	elif [ $SUCCESS_ATTEMPTS -gt 0 ]; then
		__toLog "Replication continued after check # $SUCCESS_ATTEMPTS"
		SUCCESS_ATTEMPTS=0
		MASTER_PARTIAL_MATCHES=0
	fi

done
