#!/bin/bash

CONF_FILE='/rw2/etc/pbx/trunk_notification.conf'
CALLWEAVER='/usr/sbin/callweaver'
CALLWEAVER_PID='/var/run/callweaver/callweaver.pid'
IFS="#"

if [ ! -f "$CONF_FILE" ]; then
cat << 'EOF' > $CONF_FILE
[general]
start=0
EOF
fi

[ "$(awk -F '=' '/start=/{print $2}' $CONF_FILE)" == "0" ] && exit

# check PID and uptime callweaver
[ ! -f "$CALLWEAVER_PID" ] && exit
[ "$(($(date +%s) - $(stat -c %Y $CALLWEAVER_PID)))" -le "120" ] && exit

FLAG() {
  local SIP="/tmp/sip_trunk_$(md5sum <<< "$2" | awk '{print $1}').lock"
  local STATUS="$1"; TRY_COUNT="2"

  [ -f "$SIP" ] && local GET_TRY="$(cat $SIP)"

  case "$STATUS" in
    UNSET)
      if [ "$GET_TRY" ]; then
        [ "$GET_TRY" -ge "$TRY_COUNT" ] && echo "UP"
        rm -f $SIP
      else
        rm -f $SIP
      fi
    ;;
    SET)
      if [ "$GET_TRY" ]; then
        local SET_TRY="$(($GET_TRY+1))"
        [ "$SET_TRY" -le "10" ] && echo "$SET_TRY" > $SIP
        [ "$SET_TRY" -eq "$TRY_COUNT" ] && echo "DOWN"
      else
        echo "1" > $SIP
      fi
    ;;
  esac
}

SEND_MAIL() {
  local SIP="$1"
  local MAIL_TYPE="$2"
  local MAIL_CONFIG='/etc/companies.d/0/msmtprc'
  local PBX_NAME="$(awk '/127.0.0.1/{print $4}' /etc/hosts)"
  local FROM="$(awk '/from/{print $2}' $MAIL_CONFIG)"
  local MAIL_ADMIN="$(ldapsearch -xLLL -h localhost -b "dc=wildix" -D "cn=admin,dc=wildix" -w "wildix" "(&(objectClass=wPerson)(userKind=admin)(cn=admin))" mail | awk '/mail/{print $2}')"

  # check mail password and admin mail
  [ ! "$MAIL_ADMIN" ] || [ ! "$(awk '/password/{print $2}' $MAIL_CONFIG)" ] && { logger "CHECK_SIP: send email error (not set password or admin email)"; return 404; }

  # check from
  [ ! "$FROM" ] && FROM="info@wildix.com"

  # set mail
  case "$MAIL_TYPE" in
    DOWN) local SUBJECT_STATUS="Warning"      ; local SIP_STATUS="down" ;;
      UP) local SUBJECT_STATUS="Notification" ; local SIP_STATUS="up" ;;
  esac

  # send email
  echo "$SIP" | sed 's/_/#/g' | while read HOST USERNAME NOT_USE; do
    echo -e "From: $FROM\nTo: $MAIL_ADMIN\nSubject: $SUBJECT_STATUS $PBX_NAME: $HOST ($USERNAME) is $SIP_STATUS!\n\nsip: $HOST ($USERNAME)\ntime: $(date)\nstatus: $SIP_STATUS"|\
    msmtp -C $MAIL_CONFIG "$MAIL_ADMIN"
  done
}

CHECK_SIP_TRUNKS() {
  $CALLWEAVER -rx 'sip show registry' | awk '/Registered|Request Sent/{print $1"_"$3"_"$4"#"$5"#"$6"#"$7" "$8" "$9" "$10" "$11}'| sed 's/ws2sip.wildix.com/kite-connection/g'|\
  while read SIP REFRESH STATUS LAST_REGISTRATION; do
      # check status sip registry
      case "$STATUS" in
        Registered) [ "$(FLAG UNSET $SIP)" == "UP" ] && SEND_MAIL "$SIP" "UP" ;;
           Request) [ "$(FLAG SET $SIP)" == "DOWN" ] && SEND_MAIL "$SIP" "DOWN" ;;
      esac
  done
}

CHECK_SIP_TRUNKS