#!/usr/bin/php
<?php

/* @var $certificate \Wildix\Certificate\SSLCertificate */
/* @var $log \Syslog */

require_once '/var/www/api/init.php';

$fp = fopen('/tmp/renewal_ssl_certificate.lock', 'w+');

if (!flock($fp, LOCK_EX | LOCK_NB)) {
    echo 'Lock file exists' . PHP_EOL;
    exit(0);
}

try {
    $cliOptions = getopt('fc:i:s');

    $diContainer = new \Wildix\DependencyInjection\Container();
    $certificate = $diContainer->get(\Wildix\Certificate\SSLCertificate::class);

    $attemptsCount   = $cliOptions['c'] ?? $certificate::DEFAULT_ATTEMPTS_COUNT;
    $requestInterval = $cliOptions['i'] ?? $certificate::DEFAULT_REQUEST_INTERVAL;

    $certificate->renew(isset($cliOptions['f']), $attemptsCount, $requestInterval, isset($cliOptions['s']));

    $log = $diContainer->get(\Syslog::class . ':SSLCertificateRenewal');
    updateEjabberdPem($log);

} finally {
    // remove the lock
    flock($fp, LOCK_UN);
    fclose($fp);
}

/**
 * @TODO Need remove in summer 2019. Used only for migration period for update ejabberd.pem file.
 */
function updateEjabberdPem($log) {
    $sslPrivateKeyPath  = '/etc/wildix/ssl/wildixin.com.key';
    $sslCertificatePath = '/etc/wildix/ssl/wildixin.com.crt';
    $ejabberdPemPath    = '/etc/wildix/ssl/ejabberd.pem';

    if (!is_file($sslPrivateKeyPath) || !is_file($sslCertificatePath) || !is_file($ejabberdPemPath)) {
        $log->error(__LINE__, 'Certificate file not found');
        return;
    }

    $sslPrivateKey  = file_get_contents($sslPrivateKeyPath);
    $sslCertificate = file_get_contents($sslCertificatePath);
    $ejabberdPem    = file_get_contents($ejabberdPemPath);

    if ($sslPrivateKey === false || $sslCertificate === false || $ejabberdPem === false) {
        $log->error(__LINE__, 'Can not get certificate file content');
        return;
    }

    $newEjabberdPem = $sslPrivateKey . $sslCertificate;

    if ($newEjabberdPem === $ejabberdPem) {
        $log->info(__LINE__, 'Ejabberd certificate does not need renewal');
        return;
    }

    if (file_put_contents($ejabberdPemPath, $newEjabberdPem) === false) {
        $log->error(__LINE__, 'Can not save new ejabberd certificate');
        return;
    }

    $log->info(__LINE__, 'Ejabberd certificate successfully renewed');

    chmod($ejabberdPemPath, 0640);
    chown($ejabberdPemPath, 'wms');
    chgrp($ejabberdPemPath, 'wms');

    exec('sudo /etc/init.d/ejabberd restart');
}
