#!/usr/bin/php
<?php
//error_reporting(E_ALL ^ E_NOTICE);
//ini_set('display_errors','On');

define('BASEPATH', '/var/www/');

require_once(BASEPATH.'init.php');
require_once(BASEPATH.'cticonnect/classes/cti_db.php');

function getHash(array $data){
    ksort($data);

	return md5(implode($data));
}

// init system log
$syslog = new Syslog('sync_ldap_users');

// check if upgrade procedure is running
exec('/usr/sbin/wms_can_mount_to_ro', $out, $retval);
if ($retval != '0') {
	$syslog->warning(__LINE__, 'Upgrade is currently running. Script will not start');
	exit(); // Exit due to upgrade is started
}

// check if sync already running
exec('pgrep "sync_ldap_users" -c', $output);
$result = intval(implode('', $output));
if ($result > 1) {
	// Exit due to the same script already started
	$syslog->warning(__LINE__, 'Another instance of this script is already running');
	exit();
}

$db = new \CTI\DB('/mnt/cticonnect/cticonnectdb');

//////////////////////////////////////
// get phonebook users
$query = "SELECT rid FROM cti_phonebooks WHERE type = 3";
$ldapPhonebookId = null;
if($res = $db->query($query)){
	if ($row = $res->fetchArray(SQLITE3_ASSOC)) {
		$ldapPhonebookId = $row['rid'];
	} else {
		$hash = md5('LDAP30');
		$db->exec("INSERT INTO cti_phonebooks (name, type, owner_id, hash) VALUES ('LDAP', 3, 0, '{$hash}');");
		$ldapPhonebookId = $db->lastInsertRowID();
	}
	$res->finalize();
}

if ($ldapPhonebookId === null || $ldapPhonebookId < 1){
	$syslog->warning(__LINE__, 'The phone book ID does not found');
	exit();
}

$phoneBookUsers = array();
$duplPhoneBookUsers = array();
if($res = $db->query("SELECT extension, rid, group_id, hash FROM cti_contacts WHERE phonebook_id = {$ldapPhonebookId};")){
	while ($row = $res->fetchArray(SQLITE3_ASSOC)) {
		if (isset($phoneBookUsers[$row['extension']])){
			//For a some reason in cti_contacts appears duplicated extensions
			//for now duplicated extensions just will are deleted
			$duplPhoneBookUsers[] = $row;
		} else {
			$phoneBookUsers[$row['extension']] = $row;
		}
	}
	$res->finalize();
}
$db->disconnect();
/////////////////////////////////////
////////////////////////////////////////
// get ldap users
$colleagues                    = \Wildix\Colleagues\Colleagues::getInstance();
$fieldTelephoneNumber          = $colleagues->getLdapKey(\Wildix\Colleagues\Colleague::EXTENSION);
$fieldDn                       = $colleagues->getLdapKey(\Wildix\Colleagues\Colleague::DN);
$fieldCn                       = $colleagues->getLdapKey(\Wildix\Colleagues\Colleague::NAME);
$fieldOwner                    = $colleagues->getLdapKey(\Wildix\Colleagues\Colleague::GROUP_DN);
$fieldMail                     = $colleagues->getLdapKey(\Wildix\Colleagues\Colleague::EMAIL);
$fieldFacsimileTelephoneNumber = $colleagues->getLdapKey(\Wildix\Colleagues\Colleague::FAX_NUMBER);
$fieldMobile                   = $colleagues->getLdapKey(\Wildix\Colleagues\Colleague::MOBILE_PHONE);
$fieldImageUrl                 = $colleagues->getLdapKey(\Wildix\Colleagues\Colleague::PICTURE);
$fieldWOfficePhone             = $colleagues->getLdapKey(\Wildix\Colleagues\Colleague::OFFICE_PHONE);

$ldapProps = [
        $fieldDn, $fieldCn, $fieldOwner, $fieldMail, $fieldTelephoneNumber, $fieldFacsimileTelephoneNumber, $fieldMobile, $fieldImageUrl, $fieldWOfficePhone
];

$cookie = '';
$count = 500;

$ldapConnect = WMS_Ldap::getInstance()->getConnect();
ldap_set_option($ldapConnect, LDAP_OPT_TIMELIMIT, 2);
$ldapParam = WMS_Ldap::getInstance()->getConnectParams();
$filter = "(&(objectClass=".(WMS_Ldap::PERSONS).")(|(userKind=user)(userKind=admin)(userKind=fax)(userKind=park_orbit)))";

do{
	$querys = array();

	ldap_control_paged_result($ldapConnect, $count, true, $cookie);

	$searchResult = @ldap_search($ldapConnect, $ldapParam['basedn'], $filter, $ldapProps, 0, 0, 2);
	if (!$searchResult) {
		return ldap_error($ldapConnect);
	}

	$entryID = ldap_first_entry($ldapConnect,$searchResult);

	do {
		$row = ldap_get_attributes($ldapConnect, $entryID);

		$user = array();
		foreach ($ldapProps as $attr) {

			if($attr == 'dn'){
				$user['dn'] = ldap_get_dn($ldapConnect, $entryID);
			}else{
				if(isset($row[$attr]['count'])){
					if($row[$attr]['count']==1){
						$user[$attr] = $row[$attr][0];
					}else{
						unset($row[$attr]['count']);
						$user[$attr] = $row[$attr];
					}
				}
				else{
					$user[$attr] = $row[$attr];
				}
			}
		}

        if (!isset($user[$fieldTelephoneNumber]) || !isset($user[$fieldCn])) {
			continue;
		}

		if (!isset($user[$fieldMail])) {
			$user[$fieldMail] = '';
		}
		if (!isset($user[$fieldFacsimileTelephoneNumber])) {
			$user[$fieldFacsimileTelephoneNumber] = '';
		}
		if (!isset($user[$fieldMobile])) {
			$user[$fieldMobile] = '';
		}
		if (!isset($user[$fieldImageUrl])) {
			$user[$fieldImageUrl] = '';
		}

        $userTelephoneNumber = $user[$fieldTelephoneNumber];

		$data = array(
			    'username' => $user[$fieldCn],
				'email' => $user[$fieldMail],
				'extension' => $userTelephoneNumber,
				'phone' => '',
				'office' => $user[$fieldWOfficePhone],
				'home' => '',
				'home_mobile' => '',
				'fax' => $user[$fieldFacsimileTelephoneNumber],
				'mobile' => $user[$fieldMobile],
				'organization' => 'LDAP',
				'image_url' => $user[$fieldImageUrl],
				'shortcut' => '',
				'phonebook_id' => $ldapPhonebookId,
				'group_id' => $user[$fieldOwner],
				'user_class' => '',
				'note' => '',
				'address' => '',
				'vat_id' => '',
				'name_prefix' => '',
				'pin' => '',
				'credit' => '',
				'country' => '',
				'date_birth' => '',
				'born_in' => '',
				'postal_code' => '',
				'city' => '',
				'province' => '',
				'license_plate' => '',
				'document_released_on' => '',
				'document_from' => '',
                'source_id' => $user[$fieldDn],
                'owner_id' => '',
		);

		$data['hash'] = getHash($data);

		if (isset($phoneBookUsers[$userTelephoneNumber])){
			// update
			if($phoneBookUsers[$userTelephoneNumber]['hash'] != $data['hash'] OR $phoneBookUsers[$userTelephoneNumber]['group_id'] != $user['owner']){
				$setSql = array();
				foreach($data as $key=>$value){
					$setSql[] = "{$key} = '{$db->escape($value)}'";
				}

				if(!empty($setSql)){
					$querys[] = "UPDATE cti_contacts SET ".join(", ", $setSql)." WHERE rid = {$phoneBookUsers[$userTelephoneNumber]['rid']};";
				}
			}

			unset($phoneBookUsers[$userTelephoneNumber]);
		}else{
			// insert
			$keys = array();
			$values = array();
			foreach($data as $key=>$value){
				$keys[] = $key;
				$values[] = "'{$db->escape($value)}'";
			}
			$querys[] = "INSERT INTO cti_contacts (".join(", ", $keys).") VALUES(".join(", ", $values).");";

		}

	}while ($entryID = ldap_next_entry($ldapConnect,$entryID));

	if (!empty($querys)){
		$db->connect();
		if($db->exec('BEGIN TRANSACTION;')){
			if(!$db->exec(join("\n", $querys))){
				$syslog->error(__LINE__, "Error ({$db->lastErrorCode()}): {$db->lastErrorMsg()}");
			}

			if(!$db->exec('COMMIT;')){
				$syslog->error(__LINE__, "Error ({$db->lastErrorCode()}): {$db->lastErrorMsg()}");
			}
		}else{
			$syslog->error(__LINE__, "Error ({$db->lastErrorCode()}): {$db->lastErrorMsg()}");
		}
		$db->disconnect();
	}

	ldap_control_paged_result_response($ldapConnect, $searchResult, $cookie);
	@ldap_free_result($searchResult);

} while($cookie !== null && $cookie != '');

//////////////////////////////////////
$querys = array();

$phoneBookUsers = array_merge($phoneBookUsers, $duplPhoneBookUsers);
if(!empty($phoneBookUsers)){
	$rd = array();
	foreach($phoneBookUsers as $phoneBookUser){
		$rd[] = $phoneBookUser['rid'];
	}
	$querys[] = "DELETE FROM cti_contacts WHERE rid IN ('".implode("','", $rd)."');";
}

if (!empty($querys)){
	$db->connect();
	if($db->exec('BEGIN TRANSACTION;')){
		if(!$db->exec(join("\n", $querys))){
			$syslog->error(__LINE__, "Error ({$db->lastErrorCode()}): {$db->lastErrorMsg()}");
		}

		if(!$db->exec('COMMIT;')){
			$syslog->error(__LINE__, "Error ({$db->lastErrorCode()}): {$db->lastErrorMsg()}");
		}
	}else{
		$syslog->error(__LINE__, "Error ({$db->lastErrorCode()}): {$db->lastErrorMsg()}");
	}
	$db->disconnect();
}

exit();
