#!/usr/bin/lua

local wmsldap = require"core.wmsldap"

function readFile(path)
	local content = ""
	local file, error = io.open(path , "r")
	if file then
	    content = file:read("*all")
	    io.close(file)
	end
	return content
end

local currentDomain = readFile("/etc/hostname")..".wildixin.com";
local newXmppSrvRecords = ""
local newSipNaptrRecords = ""
local newSipSrvRecords = ""
local pbx = {}
local arrayOfPbxs = {}
local arrayOfFailovers = {}
local json = require"dkjson"
local prio = 0
local weight = 1

local filter = '(objectClass=wPbx)'
local attrs ={"dn", "pbxIp", "pbxDomain", "description", "pbxPublicIp", "pbxPublicPort" }
local failoverFilter = '(objectClass=wFailover)'
local failoverAttrs ={"dn", "pbxowner", "pbxFailover", "priority", "enabled"}
local connection, basedn = wmsldap:getConection()

if connection ~= nil then
	for dn,attrs in connection:search { attrs = attrs, filter = filter, base = basedn, scope = "subtree"} do
		arrayOfPbxs[dn] = attrs
	end

	for dn,attrs in connection:search { attrs = failoverAttrs, filter = failoverFilter, base = basedn, scope = "subtree"} do
		arrayOfFailovers[dn] = attrs
	end
end
	connection:close()

for dn,attrs in pairs(arrayOfPbxs) do

		if attrs["pbxDomain"] ~= currentDomain then

-------------------- XMPP ----------------------
		  prio = 1
      newXmppSrvRecords = newXmppSrvRecords.."### SRV for " ..attrs["pbxDomain"].. " ### \n"

-- use custom secure port with Public IP
      if attrs["pbxPublicIp"] ~= nil then
        local xmppPublicPort = 443;
        if attrs["pbxPublicPort"] ~= nil then
          xmppPublicPort = attrs["pbxPublicPort"];
        end
        newXmppSrvRecords = newXmppSrvRecords.."srv-host=_xmpp-server._tcp."..attrs["pbxDomain"]..","..attrs["pbxPublicIp"]..","..xmppPublicPort..","..prio..","..weight.."\n"
        newXmppSrvRecords = newXmppSrvRecords.."srv-host=_xmpp-server._tcp.conference."..attrs["pbxDomain"]..","..attrs["pbxPublicIp"]..","..xmppPublicPort..","..prio..","..weight.."\n"
      end

-- use local interfaces with 5269 port
      if attrs["description"] then
        local a, b = string.match(attrs["description"], "(.*)%=(.*)")
        local res = json.decode(b)
        for key, ip in pairs(res["addrSet"]["list"]) do
          prio = prio+1
          newXmppSrvRecords = newXmppSrvRecords.."srv-host=_xmpp-server._tcp."..attrs["pbxDomain"]..","..ip..",5269,"..prio..","..weight.."\n"
          newXmppSrvRecords = newXmppSrvRecords.."srv-host=_xmpp-server._tcp.conference."..attrs["pbxDomain"]..","..ip..",5269,"..prio..","..weight.."\n"
        end
      end

-- n2n addr
      prio = prio+1
			newXmppSrvRecords = newXmppSrvRecords.."srv-host=_xmpp-server._tcp."..attrs["pbxDomain"]..","..attrs["pbxIp"]..",5269,"..prio..","..weight.."\n"
			newXmppSrvRecords = newXmppSrvRecords.."srv-host=_xmpp-server._tcp.conference."..attrs["pbxDomain"]..","..attrs["pbxIp"]..",5269,"..prio..","..weight.."\n"
      newXmppSrvRecords = newXmppSrvRecords.."\n"

-------------------- SIP ----------------------

-- NAPTR
      newSipNaptrRecords = newSipNaptrRecords.."### NAPTR for " .. attrs["pbxDomain"] .. " ### \n"
      newSipNaptrRecords = newSipNaptrRecords.."naptr-record="..attrs["pbxDomain"]..",0,0,S,SIPS+D2T,,_sips._tcp."..attrs["pbxDomain"].."\n"
--      newSipNaptrRecords = newSipNaptrRecords.."naptr-record="..attrs["pbxDomain"]..",1,0,S,SIP+D2T,,_sip._tcp."..attrs["pbxDomain"].."\n"
      newSipNaptrRecords = newSipNaptrRecords.."naptr-record="..attrs["pbxDomain"]..",2,0,S,SIP+D2U,,_sip._udp."..attrs["pbxDomain"].."\n"
      newSipNaptrRecords = newSipNaptrRecords.."\n"

-- SRV
      prio = 1
      newSipSrvRecords = newSipSrvRecords.."### SRV for " ..attrs["pbxDomain"].. " ### \n"

-- use custom secure port with Public IP
      if attrs["pbxPublicIp"] ~= nil then
        local sipPublicPort = 443;
        if attrs["pbxPublicPort"] ~= nil then
           sipPublicPort = attrs["pbxPublicPort"];
        end
        newSipSrvRecords = newSipSrvRecords.."srv-host=_sips._tcp."..attrs["pbxDomain"]..","..attrs["pbxPublicIp"]..","..sipPublicPort..","..prio..","..weight.."\n"
--        newSipSrvRecords = newSipSrvRecords.."srv-host=_sip._tcp."..attrs["pbxDomain"]..","..attrs["pbxPublicIp"]..","..sipPublicPort..","..prio..","..weight.."\n"
        newSipSrvRecords = newSipSrvRecords.."srv-host=_sip._udp."..attrs["pbxDomain"]..","..attrs["pbxPublicIp"]..",5060,"..prio..","..weight.."\n"
      end

-- use local interfaces with 5060 port
  		if attrs["description"] then
  		  local a, b = string.match(attrs["description"], "(.*)%=(.*)")
  		  local res = json.decode(b)
  		  for key, ip in pairs(res["addrSet"]["list"]) do
  			  prio = prio+1
			  newSipSrvRecords = newSipSrvRecords.."srv-host=_sips._tcp."..attrs["pbxDomain"]..","..ip..",5061,"..prio..","..weight.."\n"
--  			  newSipSrvRecords = newSipSrvRecords.."srv-host=_sip._tcp."..attrs["pbxDomain"]..","..ip..",5060,"..prio..","..weight.."\n"
  			  newSipSrvRecords = newSipSrvRecords.."srv-host=_sip._udp."..attrs["pbxDomain"]..","..ip..",5060,"..prio..","..weight.."\n"
  		  end
  		end

-- n2n addr
      prio = prio+1
--      newSipSrvRecords = newSipSrvRecords.."srv-host=_sip._tcp."..attrs["pbxDomain"]..","..attrs["pbxIp"]..",5060,"..prio..","..weight.."\n"
      newSipSrvRecords = newSipSrvRecords.."srv-host=_sip._udp."..attrs["pbxDomain"]..","..attrs["pbxIp"]..",5060,"..prio..","..weight.."\n"
      newSipSrvRecords = newSipSrvRecords.."\n"
		end
end


local needRestart = "no"
local srvSipRecords = readFile("/etc/dnsmasq.d/sip_dns.conf")
if srvSipRecords ~= newSipNaptrRecords..newSipSrvRecords then
	local file, error = io.open("/etc/dnsmasq.d/sip_dns.conf", "w")
	file:write(newSipNaptrRecords .. newSipSrvRecords)
	io.close(file)
	needRestart = "yes"
end

local srvXmppRecords = readFile("/etc/dnsmasq.d/ejabberd_dns.conf")
if srvXmppRecords ~= newXmppSrvRecords then
	local file, error = io.open("/etc/dnsmasq.d/ejabberd_dns.conf", "w")
	file:write(newXmppSrvRecords)
	io.close(file)
	needRestart = "yes"
end

if needRestart == "yes" then
	os.execute("sudo /etc/init.d/dnsmasq restart")
	os.execute("sudo curl http://127.0.0.1:5060/v1/wmsnet/reload/")
end

