(function universalModuleDefinition(root, factory){
    if (typeof exports == 'object'){
        // CommonJS
        module.exports = factory(require('jquery'));
    } else if (typeof define == 'function' && define.amd){
        // AMD
        define(['jquery'], factory);
    } else if (typeof jQuery !== 'undefined'){
        // Browser
        root.ActivationView = factory(root.jQuery);
    }
}(this, function ($){
    'use strict';

    var ActivationView = function(data){
        this.constructor(data);
    };

    ActivationView.prototype = {
        constructor: function(data){
            this._data = data;

            this._form = $('#licenses_form');

            this._form.on('change', 'input.license_mode', $.proxy(this._onChangeLicenseMode, this));
            this._form.on('click', 'input.btnRefreshViaInternet', $.proxy(this._onClickBtnRefreshViaInternet, this));
            this._form.on('click', 'input.btnDeactivate', $.proxy(this._onClickBtnDeactivate, this));
            this._form.on('click', 'input.btnActivate', $.proxy(this._onClickBtnActivate, this));
            this._form.on('click', 'input.btnDownloadLogo', $.proxy(this._onClickBtnDownloadLogo, this));
            this._form.on('click', 'input.btnRemoveLogo', $.proxy(this._onClickBtnRemoveLogo, this));
            this._form.on('click', 'input.btnRenewCertificate', $.proxy(this._onClickBtnRenewCertificate, this));

            this._renewCertificateDialog = $('#renewCertificateDialog').dialog({
                closeOnEscape: true,
                autoOpen: false,
                resizable: false,
                height: 250,
                width: 410
            });
        },

        load: function(){
            this._draw();
            return this;
        },

        _getLicenseValue: function(licenseName){
            if (this._data.limitations[licenseName]['state'] == true) {
                var value = WMS._t('buttons.Yes');

                if (this._data.limitations[licenseName]['until'] != '*') {
                    var date = new Date(this._data.limitations[licenseName]['until'] * 1000);
                    value += ' (' + WMS._t('strings.validUntil') + ' ' + date.format('yyyy-mm-dd') + ')';
                }
                return value;
            }
            return WMS._t('buttons.No');
        },

        _draw: function(){
            var activationTable = this._form.find('table.activation');
            activationTable.find('.state').html((this._data.activated === true ? WMS._t('license.Activated') : WMS._t('license.NotActivated')));

            activationTable.find('.hwserial').html(this._data.hwserial);
            activationTable.find('.swserial').html(this._data.swserial);

            activationTable.find('.memory').html(Math.ceil(this._data.memory / 1024 / 1024) + ' Mb');
            activationTable.find('.processor').html(this._data.processor);
            activationTable.find('.domain').html(this._data.domain);
            activationTable.find('.rooms').html(this._data.limitations['rooms']);
            activationTable.find('.phones').html(this._data.limitations['phones']);
            activationTable.find('.calls').html(this._data.limitations['calls']);

            $.each(this._data.limitations.licenseTypes, $.proxy(function(type) {
                var availableCount = (this._data.limitations.licenseTypes[type] | 0);
                var usedCount = (this._data.licenseUsage[type] | 0);

                activationTable.find('.' + type + '-licenses-count').html(availableCount);
                activationTable.find('.' + type + '-licenses-used').html(usedCount);
            }, this));

            if(this._data.limitations['rooms'] < 1){
                activationTable.find('.rooms').closest('tr').hide();
            }else{
                activationTable.find('.rooms').closest('tr').show();
            }

            var licensesTable = this._form.find('table.licenses');
            licensesTable.find('.cti_connect_pro').html(this._getLicenseValue('cti_connect_pro'));
            licensesTable.find('.cti_cdr_view').html(this._getLicenseValue('cti_cdr_view'));
            licensesTable.find('.support').html(this._getLicenseValue('support'));
            licensesTable.find('.tapi').html(this._getLicenseValue('tapi'));

            var licensesActionsTable = this._form.find('table.licenses_actions');

            if(this._data.activated){
                licensesActionsTable.find('tr').hide();

                licensesActionsTable.find('.btnActivate').hide();
                licensesActionsTable.find('.btnDeactivate').show();
            }else{
                licensesActionsTable.find('tr').show();

                licensesActionsTable.find('.btnActivate').show();
                licensesActionsTable.find('.btnDeactivate').hide();
            }

            if(this._data.limitations['cloud']['state']){
                licensesActionsTable.find('.license-file').hide();
                licensesActionsTable.find('.btnDeactivate').hide();
            }else{
                licensesActionsTable.find('.license-file').show();
                licensesActionsTable.find('.btnDeactivate').show();
            }

            licensesActionsTable.find('.buttons').show();

            if(this._form.find('input.license_mode:checked').length == 0){
                this._form.find('input.license_mode:eq(0)').prop('checked', true).change();
            }
        },

        _parseResponseError: function(jqXHR){
            var error = '';
            if(jqXHR && jqXHR.responseText){
                var response = JSON.parse(jqXHR.responseText);
                if(response && response['reason']){
                    error = response['reason'];
                }
            }else if(jqXHR && jqXHR.statusText){
                error = jqXHR.statusText;
            }
            return error;
        },

        _loadData: function(){
            $.ajax({
                url: 'api/v1/PBX/settings/license',
                type: 'GET',
                dataType: 'json',

                success: $.proxy(function (response, textStatus, jqXHR) {
                    if(response && response.type == 'result'){
                        this._data = response.result;
                        this._draw();
                    }
                }, this),
                error: $.proxy(function(jqXHR, textStatus, errorThrown){
                    setError('error', this._parseResponseError(jqXHR));
                }, this),
                complete: function(){
                    ap_showWaitMessage('waitDiv', 0);
                }
            });
        },

        _onChangeLicenseMode: function(event){
            this._form.find('input.license_mode:not(:checked)').closest('tr').find('td:eq(1) input').prop('disabled', true);
            this._form.find('input.license_mode:checked').closest('tr').find('td:eq(1) input').prop('disabled', false);
        },

        _onClickBtnRefreshViaInternet: function(event){
            ap_showWaitMessage('waitDiv', 1);
            $.ajax({
                url: 'api/v1/PBX/settings/license',
                type: 'POST',
                dataType: 'json',
                data: {
                    _METHOD: 'PUT'
                },
                success: $.proxy(function (response, textStatus, jqXHR) {
                    this._loadData();
                }, this),
                error: $.proxy(function(jqXHR, textStatus, errorThrown){
                    ap_showWaitMessage('waitDiv', 0);
                    setError('error', this._parseResponseError(jqXHR));
                }, this)
            });
        },

        _onClickBtnDeactivate: function(event){
            if (confirm(WMS._t('license.JS.DoYouWantToDeactivate'))){
                ap_showWaitMessage('waitDiv', 1);
                $.ajax({
                    url: 'api/v1/PBX/settings/license',
                    type: 'POST',
                    dataType: 'json',
                    data: {
                        _METHOD: 'DELETE'
                    },
                    success: $.proxy(function (response, textStatus, jqXHR) {
                        this._loadData();
                    }, this),
                    error: $.proxy(function(jqXHR, textStatus, errorThrown){
                        ap_showWaitMessage('waitDiv', 0);
                        setError('error', this._parseResponseError(jqXHR));
                    }, this)
                });
            }
        },
        _onClickBtnActivate: function(event){
            var data = null;

            if(this._form.find('input.license_mode:checked').val() == 'perPbxMode'){
                data = new FormData();
                data.append('serial', this._form.find('input[name=licserial]').val());
                data.append('code1', this._form.find('input[name=liccode1]').val());
                data.append('code2', this._form.find('input[name=liccode2]').val());

                var file = this._form.find('input.license_file');
                if(file.length > 0){
                    var files = file.prop('files');
                    if(files.length > 0){
                        data.append('file', files[0]);
                    }
                }
            }else{
                data = new FormData();
                data.append('key', this._form.find('input[name=pbxKey]').val());
            }

            ap_showWaitMessage('waitDiv', 1);
            $.ajax({
                url: 'api/v1/PBX/settings/license',
                type: 'POST',
                dataType: 'json',
                processData: false,
                contentType: false,
                timeout: 10 * 1000, // cw have 30s timeout
                data: data,
                success: $.proxy(function (response, textStatus, jqXHR) {
                    setTimeout(this._loadData.bind(this), 1000);
                }, this),
                error: $.proxy(function(jqXHR, textStatus, errorThrown){
                    ap_showWaitMessage('waitDiv', 0);
                    setError('error', this._parseResponseError(jqXHR));
                }, this)
            });
        },

        _onClickBtnDownloadLogo: function(event){
            ap_showWaitMessage('waitDiv', 1);
            $.ajax({
                url: 'api/v1/PBX/settings/logo',
                type: 'POST',
                dataType: 'json',
                data: {
                    _METHOD: 'PUT'
                },
                success: $.proxy(function (response, textStatus, jqXHR) {
                    if(response && response.type == 'result'){
                        $('#CustLogo').remove();
                        $('#menu').after('<div id="CustLogo"><img src="' + response.result.logo + '" title="' + WMS._t('strings.providedBy') + '"></div>');
                        this._form.find('.btnRemoveLogo').prop('disabled', false);
                    }
                }, this),
                error: $.proxy(function(jqXHR, textStatus, errorThrown){
                    setError('error', this._parseResponseError(jqXHR));
                }, this),
                complete: function(){
                    ap_showWaitMessage('waitDiv', 0);
                }
            });
        },

        _onClickBtnRemoveLogo: function(event){
            ap_showWaitMessage('waitDiv', 1);
            $.ajax({
                url: 'api/v1/PBX/settings/logo',
                type: 'POST',
                dataType: 'json',
                data: {
                    _METHOD: 'DELETE'
                },
                success: $.proxy(function (response, textStatus, jqXHR) {
                    if(response && response.type == 'result'){
                        $('#CustLogo').remove();
                        this._form.find('.btnRemoveLogo').prop('disabled', true);
                    }
                }, this),
                error: $.proxy(function(jqXHR, textStatus, errorThrown){
                    setError('error', this._parseResponseError(jqXHR));
                }, this),
                complete: function(){
                    ap_showWaitMessage('waitDiv', 0);
                }
            });
        },

        _onClickBtnRenewCertificate: function () {
            ap_showWaitMessage('waitDiv', 1);
            $.ajax({
                url: 'api/v1/PBX/settings/SSLCertificate',
                type: 'POST',
                dataType: 'json',
                data: {
                    forced: 1,
                    attemptsCount: 1
                },
                success: function () {
                    this._renewCertificateDialog.find('.proc').show();
                    this._renewCertificateDialog.find('.result-message').hide();
                    this._renewCertificateDialog.dialog('open');
                    this._checkRenewCertificateProcessStatus();
                }.bind(this),
                error: function (jqXHR) {
                    setError('error', this._parseResponseError(jqXHR));
                }.bind(this),
                complete: function(){
                    ap_showWaitMessage('waitDiv', 0);
                }
            });
        },

        _checkRenewCertificateProcessStatus: function() {
            $.ajax({
                url: 'api/v1/PBX/settings/SSLCertificate/backgroundRenewState',
                type: 'GET',
                dataType: 'json',
                success: function (response) {
                    var result = response && response.result;
                    if (!result) {
                        this._renewCertificateDialog.dialog('close');
                        setError('error', this._parseResponseError({statusText: WMS._t('strings.Error')}));
                        return;
                    }

                    if (result.processRunning) {
                        setTimeout(this._checkRenewCertificateProcessStatus.bind(this), 5000);
                        return;
                    }

                    this._renewCertificateDialog.find('.proc').hide();
                    this._renewCertificateDialog.find('.result-message')
                        .html(result.errors || 'Certificate successfully renewed')
                        .show();
                }.bind(this),
                error: function (jqXHR) {
                    setError('error', this._parseResponseError(jqXHR));
                }.bind(this)
            });
        }
    };
    return ActivationView;
}));
