/* Storage module */
(function universalModuleDefinition(root, factory){
    if (typeof exports === 'object'){
        // CommonJS
        module.exports = factory(require('jquery'));
    } else if (typeof define === 'function' && define.amd){
        // AMD
        define(['jquery'], factory);
    } else {
        // Browser
        root.Storage = factory(root.jQuery);
    }
}(this, function ($) {
    'use strict';

    function Storage() {
        this.constructor();
    }

    $.extend(Storage.prototype, {

        constructor: function () {
            this.table = $('.storageTable tbody');
            this.addShareDialogElement = $('.addShare');
            this.addShareDialog = false;
            this.availableRemoteSharesMountPoints = {};
            this.addShareServiceSelect = $('#serviceSelect');
            this.addShareNfsWindowsAvailableMountPoints = $('#serviceMountPoint');
            this.addShareSmbVersionSelect = $('#smbVersionSelect');
            this.getData();
            this.initDialog();
            this.addShareServiceSwitch();
            var body = $('body');

            this.addShareServiceSelect.on('change', function () {
                this.addShareServiceSwitch();
            }.bind(this));

            this.addShareSmbVersionSelect.on('change', function () {
                this.addWarningVersionDeprecated();
            }.bind(this));

            body.on('click', '.unMountUsb', function (el) {
                var mountPoint = $(el.target).attr('data-mount-point');
                this.unMountUsb(mountPoint);
            }.bind(this));
            body.on('click', '.initUsb', function (el) {
                var device = $(el.target).attr('data-id');
                this.initUsb(device);
            }.bind(this));
            body.on('click', '.serviceSelectBtn', function (el) {
                var partition = $(el.target).attr('data-partition');
                var mountPoint = $('.sel_' + partition).val();
                this.mountUsb(partition, mountPoint);
            }.bind(this));
            body.on('click', '.unMountSmb', function (el) {
                var mountPoint = $(el.target).attr('data-mount-point');
                this.unMountSmb(mountPoint);
            }.bind(this));
            body.on('click', '.remountSmb', function (el) {
                var shareType = $(el.target).attr('data-type');
                var mountPoint = $(el.target).attr('data-mount-point');
                this.remountShare(shareType, mountPoint);
            }.bind(this));
            body.on('click', '.smbServiceSelectBtn', function (el) {
                var shareType = $(el.target).attr('data-share-type');
                var shareId = $(el.target).attr('data-share-id');
                var sharePath = $(el.target).attr('data-share-path');
                var shareFolder = $(el.target).attr('data-share-folder');
                var mountPoint = $('.smb_' + shareId).val();
                this.mountServiceFromExistingShare(mountPoint, shareType, sharePath, shareFolder);
            }.bind(this));

            $('.addShareBtn').on('click', function () {
                this.addShareDialog.dialog('open');
            }.bind(this));

            $('#storagesContainer').show();
        },
        mountServiceFromExistingShare: function (mountPoint, shareType, sharePath, shareFolder) {
            ap_showWaitMessage('waitDiv', 1);
            $.ajax({
                url: '/api/v1/PBX/settings/storages/mountToExistingShare',
                type: 'POST',
                data: {
                    'shareType': shareType,
                    'sharePath': sharePath,
                    'mountPoint': mountPoint,
                    'shareFolder': shareFolder
                },
                cache: false,
                dataType: 'json',
                success: function (data) {
                    if(data.type === 'result') {
                        setError('success', WMS._t('devices.Mount'), 3000);
                        this.getData();
                        this.addShareDialog.dialog('close');
                    }
                }.bind(this),
                error: function (data) {
                    data = data.responseJSON;
                    setError('error', WMS._t(data.reason), 5000);
                },
                complete: function () {
                    ap_showWaitMessage('waitDiv', 0);
                }
            });
        },
        initDialog: function () {
            this.addShareDialog = this.addShareDialogElement.dialog({
                width: 390,
                autoOpen: false,
                closeOnEscape: true,
                modal: true,
                title: WMS._t('storages.AddNfsWindowsShare'),
                resizable: false,
                buttons: [{
                    text: 'Add',
                    click: this.addShare.bind(this)
                }],
                open: function () {
                },
                close: function () {
                }
            });
        },
        addShare: function () {
            if (!this.addShareNfsWindowsAvailableMountPoints.val()) {
                setError('error', 'Service is not selected!', 3000);
                return;
            }

            if(this.addShareServiceSelect.val() === 'nfs') {
                this.addNfsShare();
            } else {
                this.addWindowsShare();
            }
        },
        addNfsShare: function () {
            var mountPoint = this.addShareNfsWindowsAvailableMountPoints.val();
            var name = this.addShareDialogElement.find('#shareHost').val();
            var path = this.addShareDialogElement.find('#sharePath').val();

            ap_showWaitMessage('waitDiv', 1);
            $.ajax({
                url: '/api/v1/PBX/settings/storages/mountNfs',
                type: 'POST',
                data: {
                    'mountPoint': mountPoint,
                    'name': name,
                    'path': path
                },
                cache: false,
                dataType: 'json',
                success: function (data) {
                    if(data.type === 'result') {
                        setError('success', WMS._t('devices.Mount'), 3000);
                        this.getData();
                        this.addShareDialog.dialog('close');
                    }
                }.bind(this),
                error: function (data) {
                    data = data.responseJSON;
                    if (data.reason === 'devices.err.CannotConnectToHost'){
                        setError('error', WMS._t(data.reason, name), 5000);
                    } else if (typeof data.reason === 'object') {
                        Object.values(data.reason).forEach(function(value) {
                            setError('error', WMS._t(value), 5000);
                        });
                    } else {
                        setError('error', WMS._t(data.reason), 5000);
                    }
                },
                complete: function () {
                    ap_showWaitMessage('waitDiv', 0);
                }
            });
        },
        addWindowsShare: function () {
            var mountPoint = this.addShareNfsWindowsAvailableMountPoints.val();
            var name = this.addShareDialogElement.find('#shareHost').val();
            var path = this.addShareDialogElement.find('#sharePath').val();
            var domain = this.addShareDialogElement.find('#shareDomain').val();
            var username = this.addShareDialogElement.find('#shareUsername').val();
            var password = this.addShareDialogElement.find('#sharePassword').val();
            var smbVersion = this.addShareSmbVersionSelect.val();

            ap_showWaitMessage('waitDiv', 1);
            $.ajax({
                url: '/api/v1/PBX/settings/storages/mountSmb',
                type: 'POST',
                data: {
                    'mountPoint': mountPoint,
                    'name': name,
                    'path': path,
                    'domain': domain,
                    'username': username,
                    'password': password,
                    'smbVersion': smbVersion
                },
                cache: false,
                dataType: 'json',
                success: function (data) {
                    if(data.type === 'result') {
                        setError('success', WMS._t('devices.Mount'), 3000);
                        this.getData();
                        this.addShareDialog.dialog('close');
                    }
                }.bind(this),
                error: function (data) {
                    data = data.responseJSON;
                    if (data.reason === 'devices.err.CannotConnectToHost'){
                        setError('error', WMS._t(data.reason, name), 5000);
                    } else if (typeof data.reason === 'object') {
                        Object.values(data.reason).forEach(function(value) {
                            setError('error', WMS._t(value), 5000);
                        });
                    } else {
                        setError('error', WMS._t(data.reason), 5000);
                    }
                },
                complete: function () {
                    ap_showWaitMessage('waitDiv', 0);
                }
            });
        },
        addWarningVersionDeprecated: function () {
            if (this.addShareSmbVersionSelect.val() === '1.0') {
                let html = '<span class="warning_span" style="color:#ff0000;padding-left:10px;display:inline-block; width:100%;">' +
                    WMS._t('devices.msg.SmbVersionDeprecated', '1.0') +
                    '<br></span>';
                this.addShareSmbVersionSelect.after(html);
            } else {
                this.addShareDialogElement.find('span.warning_span').remove();
            }
        },
        addShareServiceSwitch: function () {
            if(this.addShareServiceSelect.val() === 'nfs') {
                this.addShareDialogElement.find('.shareDomainRow').css('display', 'none');
                this.addShareDialogElement.find('.shareUsernameRow').css('display', 'none');
                this.addShareDialogElement.find('.sharePasswordRow').css('display', 'none');
                this.addShareDialogElement.find('.shareSmbVersionRow').css('display', 'none');
            } else {
                this.addShareDialogElement.find('.shareDomainRow').css('display', 'inline-block');
                this.addShareDialogElement.find('.shareUsernameRow').css('display', 'inline-block');
                this.addShareDialogElement.find('.sharePasswordRow').css('display', 'inline-block');
                this.addShareDialogElement.find('.shareSmbVersionRow').css('display', 'inline-block');
            }
        },
        remountShare: function (shareType, mountPoint) {
            ap_showWaitMessage('waitDiv', 1);
            $.ajax({
                url: '/api/v1/PBX/settings/storages/remountShare',
                type: 'POST',
                data: {'shareType': shareType, 'mountPoint': mountPoint},
                cache: false,
                dataType: 'json',
                success: function (data) {
                    if(data.type === 'result') {
                        setError('success', WMS._t('devices.Mount'), 3000);
                        this.getData();
                    }
                }.bind(this),
                error: function (data) {
                    data = data.responseJSON;
                    setError('error', WMS._t(data.reason), 5000);
                },
                complete: function () {
                    ap_showWaitMessage('waitDiv', 0);
                }
            });
        },

        initUsb: function (device) {
            if(confirm(WMS._t('devices.JS.ConfirmInit'))) {
                ap_showWaitMessage('waitDiv', 1);
                $.ajax({
                    url: '/api/v1/PBX/settings/storages/initUsb',
                    type: 'POST',
                    data: {'initUsb': device},
                    cache: false,
                    dataType: 'json',
                    success: function (data) {
                        if(data.type === 'result') {
                            setError('success', WMS._t('devices.Mount'), 3000);
                            this.getData();
                        }
                    }.bind(this),
                    error: function (data) {
                        data = data.responseJSON;
                        setError('error', WMS._t(data.reason), 5000);
                    },
                    complete: function () {
                        ap_showWaitMessage('waitDiv', 0);
                    }
                });
            }
        },
        unMountSmb: function (mountPoint) {
            if (confirm(WMS._t('devices.JS.ConfirmUMount', WMS._t('devices.' + mountPoint)))) {
                ap_showWaitMessage('waitDiv', 1);
                $.ajax({
                    url: '/api/v1/PBX/settings/storages/unMountSmb',
                    type: 'POST',
                    data: {'mountPoint': mountPoint},
                    cache: false,
                    dataType: 'json',
                    success: function (data) {
                        if(data.type === 'result') {
                            setError('success', WMS._t('devices.Umount'), 3000);
                            this.getData();
                        }
                    }.bind(this),
                    error: function (data) {
                        data = data.responseJSON;
                        setError('error', WMS._t(data.reason), 5000);
                    },
                    complete: function () {
                        ap_showWaitMessage('waitDiv', 0);
                    }
                });
            }
        },

        unMountUsb: function (mountPoint) {
            if (confirm(WMS._t('devices.JS.ConfirmUMount', WMS._t('devices.' + mountPoint)))) {
                ap_showWaitMessage('waitDiv', 1);
                $.ajax({
                    url: '/api/v1/PBX/settings/storages/unMountUsb',
                    type: 'POST',
                    data: {'mountPoint': mountPoint},
                    cache: false,
                    dataType: 'json',
                    success: function (data) {
                        if(data.type === 'result') {
                            setError('success', WMS._t('devices.Umount'), 3000);
                            this.getData();
                        }
                    }.bind(this),
                    error: function (data) {
                        data = data.responseJSON;
                        setError('error', WMS._t(data.reason), 5000);
                    },
                    complete: function () {
                        ap_showWaitMessage('waitDiv', 0);
                    }
                });
            }
        },
        mountUsb: function (partition, mountPoint) {
            ap_showWaitMessage('waitDiv', 1);
            $.ajax({
                url: '/api/v1/PBX/settings/storages/mountUsb',
                type: 'POST',
                data: {
                    'mountUsb': partition,
                    'mountPoint': mountPoint
                },
                cache: false,
                dataType: 'json',
                success: function (data) {
                    if(data.type === 'result') {
                        setError('success', WMS._t('devices.Mount'), 3000);
                        this.getData();
                    }
                }.bind(this),
                error: function (data) {
                    data = data.responseJSON;
                    setError('error', WMS._t(data.reason), 5000);
                },
                complete: function () {
                    ap_showWaitMessage('waitDiv', 0);
                }
            });
        },

        getData: function () {
            ap_showWaitMessage('waitDiv', 1);
            $.ajax({
                url: '/api/v1/PBX/settings/storages/',
                type: 'GET',
                cache: false,
                dataType: 'json',
                success: function (data) {
                    if(data.type === 'result') {
                        this.dataReceived(data.result);
                    }
                }.bind(this),
                error: function (data) {
                    data = data.responseJSON;
                    setError('error', WMS._t(data.reason), 5000);
                },
                complete: function () {
                    ap_showWaitMessage('waitDiv', 0);
                }
            });
        },
        /**
         * @param {object} data Ajax response object.
         * @param {object} data.usbMounts All mounted storage info.
         * @param {object} storage Instance of Storage.
         * @param {object} storage.partitionMounts Storage partitions.
         *
         * @returns {void} void.
         */
        dataReceived: function (data) {
            this.table.empty();
            var storageTitles = [];

            $.each(data.usbMounts, function (storageKey, storage) {
                if(storage.partitionMounts.length) {
                    /**
                     * @param {string} partKey Partition key.
                     * @param {object} partition Storage partition.
                     *
                     * @returns {void} void.
                     */
                    $.each(storage.partitionMounts, function (partKey, partition) {
                        var storageName = WMS._t('devices.usb.SerialNumber', storage.vendor, storage.model, 0, storage.size);
                        var partitionTitle;

                        /**
                         * @param {object} partition.geoInfo Storage additional information.
                         * @param {object} partition.geoInfo.dfsize Storage total partition size.
                         * @param {object} partition.geoInfo.avail Storage available empty size.
                         * @param {object} partition.geoInfo.usage Storage used space (percentage).
                         * @param {object} partition.geoInfo.fstype Partition file type.
                         */
                        if(partition.geoInfo.dfsize || partition.geoInfo.avail || partition.geoInfo.usage) {
                            partitionTitle = WMS._t(
                                'devices.PartitionGeometry',
                                partition.geoInfo.fstype,
                                partition.geoInfo.dfsize,
                                partition.geoInfo.avail,
                                partition.geoInfo.usage
                            );
                        } else {
                            partitionTitle = WMS._t('devices.Type', partition.geoInfo.fstype);
                        }
                        var storageId = storage.serial + '_' + partition.name;
                        var fullName = storageName + ' - ' + WMS._t('devices.Partition') + ' ' + partition.name + ' ' + partitionTitle;
                        if($.inArray(storageId, storageTitles) === -1) {
                            this.drawStorageRow(fullName, storage, storageId, storageKey);

                            storageTitles.push(storageId);

                            if(partition.points instanceof Array === false) {
                                this.drawPossibleMounts(storageId, partition);
                            }
                        }

                        if(partition.pmpoint) {
                            this.drawMountedService(storageId, partition);
                        }
                    }.bind(this));
                } else {
                    var storageName = WMS._t('devices.usb.SerialNumber', storage.vendor, storage.model, 0, storage.size);
                    var fullName = storageName;
                    var storageId = storage.serial;
                    this.drawStorageRow(fullName, storage, storageId, storageKey);
                }
            }.bind(this));
            /**
             * @param {object} data.sambaMounts Object with Windows mounts.
             * @param {object} data.nfsMounts Object with NFS mounts.
             * @param {object} data.notMounted Not mounted (broken) services.
             * @param {object} data.availableRemoteSharesMountPoints Available mountPoints for Windows/Nfs shares.
             */

            if(!$.isEmptyObject(data.sambaMounts)) {
                $.each(data.sambaMounts, function (key, value) {
                    this.drawNfsOrSambaStorageRow('Windows', key, value);
                }.bind(this));
            }
            if(!$.isEmptyObject(data.nfsMounts)) {
                $.each(data.nfsMounts, function (key, value) {
                    this.drawNfsOrSambaStorageRow('Nfs', key, value);
                }.bind(this));
            }
            if(!$.isEmptyObject(data.notMounted)) {
                $.each(data.notMounted, function (key, value) {
                    this.drawNotMountedStorageRow(key, value);
                }.bind(this));
            }

            this.availableRemoteSharesMountPoints = data.availableRemoteSharesMountPoints;

            if(
                this.availableRemoteSharesMountPoints instanceof Array === false &&
                !$.isEmptyObject(this.availableRemoteSharesMountPoints)
            ) {
                this.addServiceSelectToSmbOrNfs();
            }
            this.updateNfsWindowsServiceSelect();
        },
        drawNotMountedStorageRow: function (service, storage) {
            var shareId = storage.serial;
            if(!shareId) {
                shareId = storage.type + storage.servername;
            }
            var shareName = storage.name + '(' + storage.serial + ')';
            if(!storage.name || !storage.serial) {
                shareName = storage.type + '/' + storage.servername;
            }

            if(!$('tr[data-id="' + shareId + '"]').length) {
                var html = '<tr data-id="' + shareId + '" data-type="' + storage.type + '">';

                html +=
                  '<td class="storageType">' + WMS._t('strings.Error') + '</td>' +
                  '<td class="storageTitle" >' + shareName + '</td>' +
                  '<td class="storageButtons"></td>';

                html += '</tr>';
                this.table.append(html);
            }

            this.drawNotMountedServiceRow(service, storage, shareId);
        },
        drawNotMountedServiceRow: function (service, storage, shareId) {
            var shareType = $('tr[data-id="' + shareId + '"]').attr('data-type');
            var html = '<tr>';

            html +=
              '<td></td>' +
              '<td class="secondTd">' +
              '<span class="titleSpan">' + WMS._t('devices.' + service) + '</span>' +
              '<button data-mount-point="' + service + '" class="but20 unMountSmb" title="Remove">' +
              '<i data-mount-point="' + service + '" class="fa fa-minus"></i>' +
              '</button>';

            if(shareType && (shareType === 'nfs' || shareType === 'smb')) {
                html +=
                  '<button data-type="' + shareType + '" data-mount-point="' + service + '" class="but20 remountSmb" title="Initialize">' +
                  '<i data-type="' + shareType + '" data-mount-point="' + service + '" class="fa fa-recycle"></i>' +
                  '</button>';
            }

            html +=
              '</td>' +
              '<td class="storageButtons"></td>';

            html += '</tr>';
            $('tr[data-id="' + shareId + '"]').after(html);
        },
        addServiceSelectToSmbOrNfs: function () {
            var availableRemoteSharesMountPoints = this.availableRemoteSharesMountPoints;
            $('.storageType[data-type="smb"]').each(function () {
                var sharePath = $(this).parent().attr('data-id');
                var shareType = $(this).parent().find('.storageType').text().toLowerCase();
                var serviceName = $(this).parent().attr('data-service-name');

                if (serviceName === '/dev/shm/var/spool/callweaver/outgoing') {
                    return;
                }

                shareType = shareType === 'windows' ? 'smb' : shareType;
                var shareId = shareType + Math.floor(Math.random() * 99999999);
                var shareFolder = false;
                if(shareType === 'smb') {
                    sharePath = sharePath.split('/');
                    shareFolder = sharePath[3];
                    sharePath = sharePath[2];
                } else {
                    sharePath = sharePath.split(':/');
                    shareFolder = sharePath[1];
                    sharePath = sharePath[0];
                }

                var html = '<tr>';
                var elements = 0;
                html +=
                  '<td class="storageType"></td>' +
                  '<td class="storageTitle">' +
                  '<select class="serviceSelect smb_' + shareId + '">';
                $.each(availableRemoteSharesMountPoints, function (key, value) {
                    if (key !== '/dev/shm/var/spool/callweaver/outgoing') {
                        elements++;
                    } else {
                        return;
                    }
                    html += '<option value="' + key + '">' + WMS._t(value) + '</option>';
                });

                html +=
                  '</select>' +
                  '<button data-share-id="' + shareId + '" data-share-type="' + shareType + '" data-share-path="' +
                  sharePath + '" data-share-folder="' + shareFolder + '" class="but20 smbServiceSelectBtn" >' +
                  '<i data-share-id="' + shareId + '" data-share-type="' + shareType + '" ' +
                  'data-share-folder="' + shareFolder + '" data-share-path="' + sharePath + '" class="fa fa-plus"></i>' +
                  '</button>' +
                  '</td>' +
                  '<td></td>';

                html += '</tr>';

                if(elements) {
                    $(this).parent().after(html);
                }
            });
        },
        drawNfsOrSambaStorageRow: function (type, service, storageName) {
            if(!$('tr[data-id="' + storageName + '"]').length) {
                var html = '<tr data-service-name="' + service + '" data-id="' + storageName + '">';

                html +=
                  '<td class="storageType" data-type="smb">' + type.toUpperCase() + '</td>' +
                  '<td class="storageTitle" >' + storageName + '</td>' +
                  '<td class="storageButtons"></td>';

                html += '</tr>';
                this.table.append(html);
            }

            this.drawNfsOrSambaServiceRow(type, service, storageName);
        },
        drawNfsOrSambaServiceRow: function (type, service, storageName) {
            var html = '<tr>';

            html +=
              '<td></td>' +
              '<td class="secondTd">' +
              '<span class="titleSpan">' + WMS._t('devices.' + service) + '</span>' +
              '<button data-mount-point="' + service + '" class="but20 unMountSmb" title="Remove">' +
              '<i data-mount-point="' + service + '" class="fa fa-minus"></i>' +
              '</button>' +
              '</td>' +
              '<td class="storageButtons"></td>';

            html += '</tr>';
            $('tr[data-id="' + storageName + '"]').after(html);
        },
        updateNfsWindowsServiceSelect: function () {
            this.addShareNfsWindowsAvailableMountPoints.empty();
            $.each(this.availableRemoteSharesMountPoints, function (key, value) {
                this.addShareNfsWindowsAvailableMountPoints.append(
                    '<option value="' + key + '">' + WMS._t(value) + '</option>'
                );
            }.bind(this));
        },
        drawStorageRow: function (fullName, storage, storageId, storageKey) {
            var html = '<tr data-id="' + storageId + '">';
            var disabled = 'disabled="disabled"';
            if(storage.initEnabled) {
                disabled = '';
            }

            html +=
              '<td class="storageType">HDD</td>' +
              '<td class="storageTitle" title="Serial: ' + storage.serial + '">' + fullName + '</td>' +
              '<td class="storageButtons">' +
              '<button ' + disabled + ' class="but20 initUsb" data-id="' + storageKey + '" onclick="" title="Initialize">' +
              '<i data-id="' + storageKey + '" class="fa fa-recycle"></i>' +
              '</button>' +
              '</td>';

            html += '</tr>';
            this.table.append(html);
        },

        /**
         *
         * @param {string} storageId Storage id to append
         * @param {object} partition Partition info
         * @param {object} partition.pmpoint MountPoint path
         *
         * @returns {void} Void.
         */
        drawMountedService: function (storageId, partition) {
            var html = '<tr>';

            html +=
              '<td></td>' +
              '<td class="secondTd">' +
              '<span class="titleSpan">' + WMS._t('devices.' + partition.pmpoint) + '</span>' +
              '<button data-mount-point="' + partition.pmpoint + '" class="but20 unMountUsb" title="Remove">' +
              '<i data-mount-point="' + partition.pmpoint + '" class="fa fa-minus"></i>' +
              '</button>' +
              '</td>' +
              '<td class="storageButtons"></td>';

            html += '</tr>';
            $('tr[data-id="' + storageId + '"]').after(html);
        },
        /**
         *
         * @param {string} storageId Storage id to append
         * @param {object} partition Partition info
         * @param {object} partition.partition Partition name (example: hdd1)
         * @param {object} partition.points Partition mountPoints
         *
         * @returns {void} Void.
         */
        drawPossibleMounts: function (storageId, partition) {
            var html = '<tr>';

            html +=
              '<td class="storageType"></td>' +
              '<td class="storageTitle">' +
              '<select class="serviceSelect sel_' + partition.partition + '">';
            $.each(partition.points, function (key, value) {
                html += '<option value="' + key + '">' + WMS._t(value) + '</option>';
            });

            html +=
              '</select>' +
              '<button data-partition="' + partition.partition + '" class="but20 serviceSelectBtn" >' +
              '<i data-partition="' + partition.partition + '" class="fa fa-plus"></i>' +
              '</button>' +
              '</td>' +
              '<td></td>';

            html += '</tr>';
            $('tr[data-id="' + storageId + '"]').after(html);
        }
    });

    return Storage;
}));
