function CallGroups(core) {
    this.client = core.client;
}
const intermediateHandler = (client, options) => {
    return new Promise((resolve, reject) => {
        client(options)
            .then((res) => {
                resolve(res.data);
            })
            .catch((e) => {
                reject(e);
            });
    });
};
function CustomError(reason) {
    Error.call(this, reason);
    this.data = {
        errors: reason
    };
    if (Error.captureStackTrace) {
        Error.captureStackTrace(this, CustomError);
    } else {
        this.stack = new Error().stack;
    }
}
const callGroupUserMapperFromApi = ([id, value]) => {
    return {
        id, value
    };
};

CustomError.prototype = Object.create(Error.prototype);
CustomError.prototype.constructor = CustomError;
const path = '/v1/Dialplan/CallGroups';

Object.assign(CallGroups.prototype, {
    namespace: 'PBX.dialplans.callGroups',
    get(params) {
        return this.client.send('GET', path, params);
    },
    fetchUsers(items) {
        return intermediateHandler(this.client._transport, {
            url: 'dialplan/index.php',
            method: 'GET',
            params: {
                'action': 'getMembersByIdF',
                'vals': items
            },
            transformResponse: [
                function(data) {
                    const response = JSON.parse(data);
                    if (response.error) {
                        throw new CustomError(response.error);
                    }
                    let total = response.count;
                    delete response.count;
                    return {
                        type: 'result',
                        result: {
                            records: Object.entries(response).map(item => callGroupUserMapperFromApi(item)),
                            total: total
                        }
                    };
                }
            ]
        });
    },
    create(params) {
        return this.client.send('POST', path, params);
    },
    update(id, params) {
        return this.client.send('PUT', `${path}/${id}/`, params);
    },
    remove(id) {
        return this.client.send('DELETE', `${path}/${id}/`);
    }
});

export default CallGroups;
