import Vue from 'vue';
import Vuex from 'vuex';
import cloneDeep from '@/modules/clone-object';
import dialplanRules from './modules/dialplanRules';
import {SETTINGS_UTILITIES_GENERATE_TRACE} from '@/configuration/acl';
import {CONFIG} from './constants';

Vue.use(Vuex);

const wmsConfig = Object.assign({}, CONFIG, cloneDeep(window.wmsConfig), {env: process.env.NODE_ENV});

/* Need to change and delete */
wmsConfig.acl[SETTINGS_UTILITIES_GENERATE_TRACE] = true;

export default new Vuex.Store({
    strict: process.env.NODE_ENV !== 'production',
    namespaced: true,
    modules: {
        dialplanRules
    },
    state: {
        config: wmsConfig,
        loader: 0,
        loaderEnabled: true
    },
    mutations: {
        changeConfig(state, data) {
            state.config = data;
        },
        showLoader(state) {
            if (state.loaderEnabled) {
                let current = this.getters.getLoader();
                state.loader = ++current;
            }
        },
        hideLoader(state) {
            let current = this.getters.getLoader();
            if (current > 0) {
                state.loader = --current;
            }
        },
        enableLoader(state) {
            state.loaderEnabled = true;
        },
        disableLoader(state) {
            state.loaderEnabled = false;
        },
        resetLoader(state) {
            state.loader = 0;
        }
    },
    actions: {
        showLoader({commit}) {
            commit('showLoader');
        },
        hideLoader({commit}) {
            commit('hideLoader');
        }
    },
    getters: {
        getLoader: state => () => state.loader,
        getConfig: state => () => state.config,
        isCloud: state => () => state.config.isCloud,
        cpuInfo: state => () => state.config.cpuInfo,
        isSlave: state => () => state.config.isSlave,
        getUser: state => () => state.config.user,
        getNetworkInterfaces: state => () => state.config.networkInterfaces,
        acl: state => state.config.acl,
        isBackupsMounted: state => () => state.config.isBackupsMounted,
        isPowerCpu: state => () => state.config.cpuInfo.isPower,
        isSubscription: state => () => state.config.isSubscription,
        isPremium: state => () => state.config.isPremium,
        bTicinoIsInstall: state => () => Boolean(state.config.integrations.bTicino.installed)
    }
});
